<?php

/**
 * @file
 */

namespace Drupal\content_snapshot\Normalizer;

use Drupal\content_snapshot\Encoder\YamlEncoder;
use Drupal\content_snapshot\Logger\LoggerInterface;
use Drupal\content_snapshot\Snapshot\TranslatableSnapshotItemInterface;
use Drupal\Core\Entity\ContentEntityInterface;
use Drupal\Core\Entity\EntityFieldManagerInterface;
use Drupal\Core\Entity\EntityTypeManagerInterface;
use Drupal\Core\Entity\EntityTypeRepositoryInterface;
use Drupal\Core\Extension\ModuleHandlerInterface;
use Drupal\Core\TypedData\TypedDataInternalPropertiesHelper;
use Drupal\serialization\Normalizer\ContentEntityNormalizer as BaseContentEntityNormalizer;

/**
 * Class ContentEntityNormalizer
 */
class ContentEntityNormalizer extends BaseContentEntityNormalizer {

  use ContentSnapshotNormalizerContextTrait;

  protected $format = YamlEncoder::FORMAT;

  /**
   * @var \Drupal\content_snapshot\Logger\LoggerInterface
   */
  private $logger;

  /**
   * @var \Drupal\Core\Extension\ModuleHandlerInterface
   */
  private $moduleHandler;

  /**
   * ContentEntityNormalizer constructor.
   *
   * @param \Drupal\Core\Entity\EntityTypeManagerInterface $entity_type_manager
   * @param \Drupal\Core\Entity\EntityTypeRepositoryInterface $entity_type_repository
   * @param \Drupal\Core\Entity\EntityFieldManagerInterface $entity_field_manager
   */
  public function __construct(
    EntityTypeManagerInterface $entity_type_manager,
    EntityTypeRepositoryInterface $entity_type_repository,
    EntityFieldManagerInterface $entity_field_manager) {
    parent::__construct($entity_type_manager, $entity_type_repository, $entity_field_manager);
  }

  /**
   * {@inheritdoc}
   *
   * @param $entity ContentEntityInterface
   */
  public function normalize($entity, $format = NULL, array $context = []) {

    $this->validateContext($context);

    $attributes = [];
    /** @var \Drupal\Core\Entity\Entity $entity */
    foreach (TypedDataInternalPropertiesHelper::getNonInternalProperties($entity->getTypedData()) as $name => $field_items) {
      $attributes[$name] = $this->serializer->normalize($field_items, $format, $context);
    }

    // If "path" property is affected by base field override, it will get the
    // "internal" status for some reason. We need to get around this.
    if (!isset($attributes['path']) && $entity->hasField('path') && $entity->getFieldDefinition('path')->getType() === 'path') {
      $attributes['path'] = $this->serializer->normalize($entity->get('path'), $format, $context);
    }

    $this->cleanupNormalization($attributes, $entity);

    if ($entity->isTranslatable()) {

      $translatableFields = $entity->getTranslatableFields();
      $translatableFieldList = array_keys($translatableFields);

      $languages = $entity->getTranslationLanguages(FALSE);

      if (!empty($languages)) {
        $attributes[TranslatableSnapshotItemInterface::TRANSLATIONS_KEY] = [];
        $translationsAttributes = &$attributes[TranslatableSnapshotItemInterface::TRANSLATIONS_KEY];

        foreach ($languages as $langcode => $language) {

          $translation = $entity->getTranslation($langcode);
          $translationAttributes = [];

          foreach (TypedDataInternalPropertiesHelper::getNonInternalProperties($translation->getTypedData()) as $name => $fieldItems) {
            // If we are dealing with translation, then we want to return only translatable fields.
            if (!isset($translatableFieldList) || empty($translatableFieldList) || in_array($name, $translatableFieldList, TRUE)) {
              $translationAttributes[$name] = $this->serializer->normalize($fieldItems, $format, $context);
              unset($translationAttributes['langcode'], $translationAttributes['default_langcode']);
            }
          }

          // If "path" property is affected by base field override, it will get the
          // "internal" status for some reason. We need to get around this..
          if (!isset($translatableFieldList) || empty($translatableFieldList) || in_array('path', $translatableFieldList, TRUE)) {
            if (!isset($translationAttributes['path']) && $entity->hasField('path') && $entity->getFieldDefinition('path')->getType() === 'path') {
              $translationAttributes['path'] = $this->serializer->normalize($translation->get('path'), $format, $context);
            }
          }

          $translationsAttributes[$langcode] = $translationAttributes;
        }
      }
    }

    return $attributes;
  }

  /**
   * {@inheritdoc}
   */
  public function denormalize($data, $class, $format = NULL, array $context = []) {

    $this->validateContext($context);

    if (!empty($data[TranslatableSnapshotItemInterface::TRANSLATIONS_KEY])) {
      $translations = $data[TranslatableSnapshotItemInterface::TRANSLATIONS_KEY];
      unset($data[TranslatableSnapshotItemInterface::TRANSLATIONS_KEY]);
    }

    /** @var ContentEntityInterface $entity */
    $entity = parent::denormalize($data, $class, $format, $context);

    if (isset($translations)) {
      foreach ($translations as $langcode => $translation) {
        $translatedEntity = $entity->addTranslation($langcode);

        // We need to denormalize our translation fields first.
        foreach ($translation as $fieldName => $translatedField) {

          $fieldItemList = $translatedEntity->get($fieldName);
          // Value may be automatically set on translation creation. We need to
          // make sure it's empty because new, denormalized values won't
          // replace it, but will be appended.
          $fieldItemList->setValue(NULL);
          $this->serializer->denormalize($translatedField, get_class($fieldItemList), $format, ['target_instance' => $fieldItemList] + $context);
        }

      }
    }

    return $entity;
  }


  /**
   * @param array $attributes
   * @param \Drupal\Core\Entity\ContentEntityInterface $contentEntity
   */
  private function cleanupNormalization(array &$attributes, ContentEntityInterface $contentEntity): void {
    if ($revisionKey = $contentEntity->getEntityType()->getKey('revision')) {
      unset($attributes[$revisionKey]);
    }
  }
}
